@@ -1,5 +1,5 @@
 # prime/prime.rb
-# $Id: prime.rb,v 1.7 2004/03/25 17:19:49 komatsu Exp $
+# $Id: prime.rb,v 1.9 2004/04/29 21:35:54 komatsu Exp $
 #
 # Copyright (C) 2002, 2003, 2004 Hiroyuki Komatsu <komatsu@taiyaki.org>
 #     All rights reserved.
@@ -213,7 +213,7 @@ class Prime
     results_conversion = PrimeWordList::merge_with_label(@context,
                                                          words_overall,
                                                          words_japanese)
-    return PrimeWordList.new(words_compact | results_conversion)
+    return PrimeWordList::concat(words_compact | results_conversion)
   end
 
   def lookup_prefix (string)
@@ -296,9 +296,14 @@ class Prime
     if @cache_compact_pattern == string then
       return @cache_compact
     end
-    expands = PrimeTypeConv::expand(string)
-    query   = PrimeQuery.new(expands, nil, :prefix, @context)
-    words_compact = search(query)[0,1]
+    words_prefix    = search_prefix(string)
+    words_uniclause = search_japanese_uniclause(string)
+
+    words = PrimeWordList::merge_with_label(@context, 
+                                            words_prefix[0,1],
+                                            words_uniclause)
+
+    words_compact = words[0,1]
 
     if words_compact.length > 0 then
       predict_with_multi_clauses!(words_compact)
@@ -344,27 +349,7 @@ class Prime
   end
 
   def predict_with_multi_clauses!(results)
-    if results.empty? then
-      string.decrease {|string2|
-        expands = PrimeTypeConv::expand(string2)
-        if expands.length > 2 then
-          next
-        end
-        query = PrimeQuery.new(expands, nil, :exact, @context)
-        word1 = search(query).first
-        
-        if word1.non_nil? then
-          expands = PrimeTypeConv::expand(string[(string2.length)..-1])
-          query = PrimeQuery.new(expands, nil, :context, word1.literal)
-          word2 = search(query).first
-          if word2.non_nil? then
-            results.push(_merge_words(word1, word2))
-            break
-          end
-        end
-      }
-      word = PrimeWordList::merge_with_label(@context, results)[0,1]
-    else
+    unless results.empty? then
       word1 = results.first
       query = PrimeQuery.new([""], nil, :context, word1.literal)
       word2 = search(query).first
@@ -453,6 +438,27 @@ class PrimeWord
 end
 
 class PrimeWordList < Array
+  def PrimeWordList::concat(*words)
+    ## This method does not sort the given words.
+    result = words.flatten.compact
+
+    mark = {}
+    merged = PrimeWordList.new
+    result.each {|word|
+      if mark[word.index] then
+        word0 = mark[word.index]
+        ## FIXME: Make this merging more intelligence.
+        ## FIXME: <komatsu@taiyaki.org> (2004-01-24)
+        word0.category   = (word0.category   or word.category)
+        word0.annotation = (word0.annotation or word.annotation)
+      else
+	mark[word.index] = word
+	merged.push(word)
+      end
+    }
+    return merged
+  end
+
   def PrimeWordList::merge (*words)
     ## merge should be sorted by inter-engine score.
     ## FIXME: Isn't it necessary to sort here?
