@@ -1,5 +1,5 @@
 # taiyaki.rb: Ruby Libraries by Hiroyuki Komatsu
-# $Id: taiyaki.rb,v 1.2.2.11 2004/02/05 11:39:32 komatsu Exp $
+# $Id: taiyaki.rb,v 1.6 2004/03/25 17:19:49 komatsu Exp $
 #
 # Copyright (C) 2002 Hiroyuki Komatsu <komatsu@taiyaki.org>
 #     All rights reserved.
@@ -20,6 +20,26 @@ rescue LoadError
   end
 end
 
+### Considering under compatibility with Ruby1.6
+begin
+  Array.new([])
+rescue TypeError
+  class Array
+    alias original_initialize initialize
+
+    def initialize(value = 0)
+      if value.kind_of?(Array) then
+        original_initialize(value.length)
+        self.length.times {|i|
+          self[i] = value[i]
+        }
+      else
+        original_initialize(value)
+      end
+    end
+  end
+end
+
 def require (name, package = nil, url = nil)
   package_name =
          {'progressbar'                => 'progressbar',
